<?php
namespace Controller;

use Zeedhi\DevOps\Util\Environment;
use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request\DataSet as RequestDataset;
use Zeedhi\Framework\DTO\Request\Filter as RequestFilter;
use Service\Package as PackageService;

class Package extends \Generated\Controller\Package {
    protected $packageService;
    /**
     * @var Environment
     */
    private $environment;

    /**
     * @param Manager $dataSourceManager
     * @param Environment $environment
     * @param PackageService $packageService
     */
    public function __construct(Manager $dataSourceManager,  Environment $environment, PackageService $packageService) {
        $this->environment = $environment;
        $this->packageService = $packageService;
        parent::__construct($dataSourceManager);
    }

    /**
     * @param RequestDataset $request
     * @param DTO\Response $response
     * @param bool $showMessage
     */

    public function save(DTO\Request $request, DTO\Response $response, $showMessage = true) {
        $dataSet     = $request->getDataSet();
        $rows = $dataSet->getRows();
        $organizationId = $this->environment->getOrganizationId();
        foreach ($rows as &$row) {
            $row['ORGANIZATION_ID'] = $organizationId;
        }
        $showMessage = Utilities::verifyShowMessage($dataSet);
        $dataSet = new DataSet($dataSet->getDataSourceName(),$rows);
        $request = new RequestDataset($dataSet,$request->getMethod(),$request->getRoutePath(),$request->getUserId());
        parent::save($request, $response, $showMessage);

        $package = $dataSet->getRows();
        $package = $package[0];
        if(!empty($package['PRODUCT_ID']) && !empty($package['UPDATE_RELEASES'])) {
            $this->packageService->updateReleasePackagesByProduct($package['PRODUCT_ID']);
        }
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response){
        $request->getFilterCriteria()->addCondition("ORGANIZATION_ID", "=", $this->environment->getCurrentOrganizationId());
        parent::find($request, $response);
    }


    public function delete(DTO\Request $request, DTO\Response $response, $showMessage = true) {
        $request     = $this->buildDataSetIfRow($request);
        $dataSet     = $request->getDataSet();
        $showMessage = Utilities::verifyShowMessage($dataSet);
        parent::delete($request, $response, $showMessage);

        $package = $dataSet->getRows();
        $package = $package[0];
        if(!empty($package['PRODUCT_ID']) && !empty($package['UPDATE_RELEASES'])) {
            $this->packageService->updateReleasePackagesByProduct($package['PRODUCT_ID']);
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function findPackageByProductVersion(RequestFilter $request, Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $productVersionId = empty($conditions[0]) ? 0 : $conditions[0]['value'];
        try {
            $registers = $this->packageService->findPackageByProductVersion($productVersionId);
            $response->addDataSet(new DataSet('packageByProductVersion', $registers));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }
}

