<?php
namespace Controller;

use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\DevOps\Util\Environment;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;

class ProductFiles extends \Generated\Controller\ProductFiles {

    private $environment;
    /**
     * @var \Service\ProductFiles
     */
    private $productFilesService;

    public function __construct(Manager $dataSourceManager, Environment $environment, \Service\ProductFiles $productFilesService) {
        parent::__construct($dataSourceManager);
        $this->environment = $environment;
        $this->productFilesService = $productFilesService;
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response){
        $filterCriteria = $request->getFilterCriteria();
        $conditions = $filterCriteria->getConditions();
        $releaseId = Utilities::getValueInFilterDataSourceByName($conditions,'RELEASE_ID',true);
        $subReleaseId = Utilities::getValueInFilterDataSourceByName($conditions,'SUB_RELEASE_ID',true);
        $typeEvent = Utilities::getValueInFilterDataSourceByName($conditions,'TYPE_EVENT',true);
        $productServerId = Utilities::getValueInFilterDataSourceByName($conditions,'PRODUCT_SERVER_ID',true);
        $filterCriteria = new FilterCriteria($filterCriteria->getDataSourceName(),$conditions,$filterCriteria->getPage(),$filterCriteria->getPageSize());
        $filterCriteria->addCondition("ORGANIZATION_ID", "=", $this->environment->getCurrentOrganizationId());
        $request = new DTO\Request\Filter($filterCriteria,$request->getMethod(),$request->getRoutePath(),$request->getUserId());
        parent::find($request, $response);
        if($typeEvent){
            $rows = $response->getDataSets()[0]->getRows();
            $rows = $this->productFilesService->setMnemonicsInScript($rows,$typeEvent,$releaseId,$productServerId,$subReleaseId);
            $response->addDataSet(new DataSet('product_files', $rows));
        }

    }

    public function validateDeployServerMnemonic(DTO\Request\Filter $request, DTO\Response $response)
    {
        $conditions = $request->getFilterCriteria()->getConditions();
        $serverIds = Utilities::getValueInFilterDataSourceByName($conditions, 'SERVER_IDS');
        $deployId = Utilities::getValueInFilterDataSourceByName($conditions, 'DEPLOY_ID');
        $typeEvent = Utilities::getValueInFilterDataSourceByName($conditions, 'TYPE_EVENT');
        $productFiles = $this->productFilesService->validateDeployServerMnemonic($serverIds, $deployId, $typeEvent);
        $response->addDataSet(new DataSet('validateDeployServerMnemonic', $productFiles));
    }

    /**
     * @param DTO\Request\DataSet $request
     * @param DTO\Response $response
     * @param bool $showMessage
     */
	public function save(DTO\Request $request, DTO\Response $response, $showMessage = true)
    {
        try {
            $row              = $request->getDataSet()->getRows()[0];
            $productFileId    = $row['ID'];
            $name             = $row['NAME'];
            $productId        = $row['PRODUCT_ID'];
            $fileName         = $row['FILE_NAME'];
            $text             = $row['TEXT'];
            $relativePath     = $row['RELATIVE_PATH'];
            $type             = $row['TYPE'];
            $isRequired       = $row['IS_REQUIRED'];
            $parentId         = isset($row['PARENT_ID']) ? $row['PARENT_ID'] : null;
            $productVersionId = isset($row['PRODUCT_VERSION_ID']) ? $row['PRODUCT_VERSION_ID'] : null;
            $isWindows        = isset($row['IS_WINDOWS']) ? $row['IS_WINDOWS'] : 'N';
            $state            = $this->productFilesService->validationScriptApproved($productFileId, $text, $row['STATE']);;
            $result = $this->productFilesService->saveProductFile($productFileId, $name, $productId, $fileName, $text,
                $relativePath, $type, $isRequired, $isWindows, $parentId, $state, $productVersionId);
            $response->addDataSet(new DataSet('product_files', array(array('ID' => $result['ID']))));
            $response->addDataSet(new DataSet('error', [$result['MNEMONIC_VALIDATION']]));
        } catch (\Exception $ex) {
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function findMnemonicByProductFiles(DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $productFileId = Utilities::getValueInFilterDataSourceByName($conditions,'PRODUCT_FILE_ID', true);
        $filter = Utilities::createQueryFiltered($conditions);
        $productFiles = $this->productFilesService->findMnemonicByProductFiles($productFileId, $filter);
        $response->addDataSet(new DataSet('findMnemonicByProductFiles', $productFiles));
    }


    public function saveMnemonicByProductFiles(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            $dataSet      = $request->getDataSet();
            $row          = $dataSet->getRows()[0];
            $mnemonicId   = $row['ID'];
            $label        = $row['LABEL'];
            $defaultValue = $row['DEFAULT_VALUE'];
            $isEncrypted = $row['IS_ENCRYPTED'];
            $mnemonicGrouping = empty($row['MNEMONIC_ID']) ? null : $row['MNEMONIC_ID'];
            
            $mnemonic = $this->productFilesService->saveMnemonicForProduct($mnemonicId, $label, $defaultValue, $isEncrypted, $mnemonicGrouping);
            $response->addDataSet(new DataSet('mnemonic', [$mnemonic]));
        } catch (\Exception $ex) {
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function delete(DTO\Request $request, DTO\Response $response, $showMessage = true) {
        $request     = $this->buildDataSetIfRow($request);
        $dataSet     = $request->getDataSet();
        $showMessage = Utilities::verifyShowMessage($dataSet);
        parent::delete($request, $response, $showMessage);
    }

    public function findProductFilesToExecuteByVersion(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $conditions       = $request->getFilterCriteria()->getConditions();
            $productId        = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_ID', true);
            $productVersionId = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_VERSION_ID', true);
            $allowEmptyFilter = Utilities::getValueInFilterDataSourceByName($conditions, 'ALLOW_EMPTY_FILTER', true);
            $productFiles     = $this->productFilesService->findProductFilesToExecuteByVersion($productId, $productVersionId, $allowEmptyFilter);
            $response->addDataSet(new DataSet('findProductFilesToExecuteByVersion', $productFiles));
        } catch (\Exception $ex) {
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function findVersionProductFiles(DTO\Request\Filter $request, DTO\Response $response){
        try {
            $conditions       = $request->getFilterCriteria()->getConditions();
            $productId        = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_ID', true);
            $productVersionId = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_VERSION_ID', true);
            $typeEvent        = Utilities::getValueInFilterDataSourceByName($conditions, 'TYPE_EVENT',true);
            $productServerId  = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_SERVER_ID',true);
            $productFiles     = $this->productFilesService->findVersionProductFiles($productId, $productVersionId, $typeEvent, $productServerId, $conditions);
            $response->addDataSet(new DataSet('product_files', $productFiles));
        } catch (\Exception $ex) {
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }
}